@@ -24,7 +24,7 @@ use Getopt::Long;
 
 # General script information:
 use constant NAME    => basename($0, '.pl');       # Script name.
-use constant VERSION => '2.3.0';                   # Script version.
+use constant VERSION => '2.3.1';                   # Script version.
 
 # General script settings:
 our $HOMEDIR         = $ENV{HOME}          || $ENV{USERPROFILE} || '.';
@@ -37,6 +37,7 @@ our $with_id         = 1;                          # Include ID?
 our $with_date       = 1;                          # Include due date?
 our $with_pri        = 1;                          # Include priority?
 our $with_state      = 1;                          # Include state?
+our $with_stats      = 0;                          # Include statistics?
 our $preserve        = 0;                          # Preserve style sheet?
 our $inline          = 0;                          # Embed the style sheet?
 our $bare            = 0;                          # Leave out the HTML
@@ -189,6 +190,48 @@ table.todo_tasks .todo_finished {
 table.todo_tasks .todo_finished:hover {
   background-color: #00ff7f;
 }
+
+table.todo_stats {
+  width: 100%;
+  margin: auto;
+}
+
+table.todo_stats tr:hover {
+  background-color: #f5f5f5;
+}
+
+table.todo_stats .todo_group {
+  width: 20%;
+  text-align: left;
+}
+
+table.todo_stats .todo_progress {
+  width: 80%;
+  text-align: left;
+  font-size: x-small;
+}
+
+table.todo_stats div.todo_greenbar {
+  margin: 0 5px 0 0;
+  float: left;
+  display: block;
+  background-color: #98fb98;
+}
+
+table.todo_stats div.todo_greenbar:hover {
+  background-color: #00ff7f;
+}
+
+table.todo_stats div.todo_bluebar {
+  margin: 0 5px 0 0;
+  float: left;
+  display: block;
+  background-color: #729fcf;
+}
+
+table.todo_stats div.todo_bluebar:hover {
+  background-color: #3465a4;
+}
 END_CSS_TASKS
 
 # Set up the __WARN__ signal handler:
@@ -305,6 +348,234 @@ sub get_stats {
   return $groups, $tasks, $undone;
 }
 
+# Create a style sheet file and return the LINK element:
+sub write_style_sheet {
+  # Do not create the style sheet when writing to STDOUT:
+  return 0 if $outfile eq '-';
+
+  # Derive the style sheet file name:
+  (my $file = $outfile) =~ s/(\.html?|\.php|)$/.css/;
+
+  # Do not rewrite existing style sheet if its preservation is requested:
+  unless (-e $file && $preserve) {
+    # Open the file for writing:
+    if (open(STYLE, ">$file")) {
+      # Write the document structure related part:
+      print STYLE $css_structure unless $bare;
+
+      # Write the tasks related part:
+      print STYLE $css_tasks;
+
+      # Close the file:
+      close(STYLE);
+    }
+    else {
+      # Report failure:
+      print STDERR "Unable to write to `$file'.\n";
+
+      # Return failure:
+      return 0;
+    }
+  }
+
+  # Return the LINK element:
+  return "<link rel=\"stylesheet\" href=\"" . basename($file) .
+         "\" type=\"text/css\">\n";
+}
+
+# Return the style sheet or a LINK element pointing to it:
+sub compose_style_sheet {
+  # Check whether to have style sheet as a separate file:
+  unless ($inline) {
+    # Return the LINK element:
+    return write_style_sheet();
+  }
+  else {
+    # Return the style sheet:
+    return << "END_STYLE_SHEET";
+<style type="text/css"><!--
+$css_structure$css_tasks  --></style>
+END_STYLE_SHEET
+  }
+}
+
+# Return the HTML header:
+sub compose_html_header {
+  my ($NAME, $VERSION) = (NAME, VERSION);
+
+  # Initialize required variables:
+  my $style_sheet      = compose_style_sheet() || "\n";
+  my $timestamp        = localtime(time);
+
+  # Return the HTML document beginning:
+  return << "END_HTML_HEADER";
+<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01//EN"
+                      "http://www.w3.org/TR/html4/strict.dtd">
+<html>
+<head>
+  <meta http-equiv="Content-Type" content="text/html; charset=$encoding">
+  <meta name="Generator" content="$NAME $VERSION">
+  <meta name="Date" content="$timestamp">
+  $style_sheet  <title>$heading</title>
+</head>
+
+<body>
+
+<div id="wrapper">
+
+<div id="heading">
+  <table>
+    <tr>
+      <td>
+        <h1><a href="#">$heading</a></h1>
+        $timestamp
+      </td>
+    </tr>
+  </table>
+</div>
+
+<div id="content">
+
+END_HTML_HEADER
+}
+
+# Return the HTML footer:
+sub compose_html_footer {
+  my ($NAME, $VERSION) = (NAME, VERSION);
+
+  # Return the HTML document closing:
+  return << "END_HTML_FOOTER";
+</div>
+
+<div id="footer">
+  Generated using <a href="http://w2do.blackened.cz/">$NAME $VERSION</a>.
+</div>
+
+</div>
+
+</body>
+</html>
+END_HTML_FOOTER
+}
+
+# Return the group header:
+sub compose_group_header {
+  my ($group, $tasks, $done) = @_;
+
+  # Prepare the group statistics:
+  my $stats = sprintf "%d task%s, %d unfinished",
+                      $tasks, (($tasks != 1) ? 's' : ''),
+                      $tasks - $done;
+
+  # Prepare the table header columns:
+  my $cols = '';
+  $cols .= "    <th class=\"todo_id\">id</th>\n" if $with_id;
+  $cols .= "    <th class=\"todo_date\">due date</th>\n" if $with_date;
+  $cols .= "    <th class=\"todo_priority\">priority</th>\n" if $with_pri;
+  $cols .= "    <th class=\"todo_state\">state</th>\n" if $with_state;
+  $cols .= "    <th class=\"todo_description\">task</th>";
+
+  # Return the group heading and the table header:
+  return << "END_GROUP_HEADER";
+<h2 class="todo_group">
+  <a name="$group">$group</a>
+  <span class="todo_stats">$stats</span>
+</h2>
+
+<table class="todo_tasks">
+  <tr>
+$cols
+  </tr>
+END_GROUP_HEADER
+}
+
+# Return the group footer:
+sub compose_group_footer {
+  # Return the table closing:
+  return "</table>\n\n";
+}
+
+# Return the task entry:
+sub compose_task_entry {
+  my ($id, $date, $priority, $state, $task) = @_;
+
+  # Decide which class the task belongs to:
+  my $class      = ($state eq 't') ? ' class="todo_finished"' : '';
+
+  # Prepare the aliases:
+  my @priorities = ('very high', 'high', 'medium', 'low', 'very low');
+  $state         = ($state eq 't') ? 'ok' : '&nbsp;';
+  $priority      = $priorities[--$priority];
+
+  # Escape reserved characters:
+  $task =~ s/&/&amp;/g;
+  $task =~ s/</&lt;/g;
+  $task =~ s/>/&gt;/g;
+
+  # Prepare the table columns:
+  my $cols = '';
+  $cols .= "    <td class=\"todo_id\">$id</td>\n" if $with_id;
+  $cols .= "    <td class=\"todo_date\">$date</td>\n" if $with_date;
+  $cols .= "    <td class=\"todo_priority\">$priority</td>\n" if $with_pri;
+  $cols .= "    <td class=\"todo_state\">$state</td>\n" if $with_state;
+  $cols .= "    <td class=\"todo_description\">$task</td>";
+
+  # Return the task entry:
+  return << "END_TASK_ENTRY";
+  <tr$class>
+$cols
+  </tr>
+END_TASK_ENTRY
+}
+
+# Return the statistics header:
+sub compose_statistics_header {
+  my ($groups, $tasks, $undone) = @_;
+
+  # Prepare the overall statistics:
+  my $stats = sprintf "%d group%s, %d task%s, %d unfinished",
+                      $groups, (($groups != 1) ? 's' : ''),
+                      $tasks,  (($tasks  != 1) ? 's' : ''),
+                      $undone;
+  # Return the statistics heading and the table header:
+  return << "END_STATISTICS_HEADER";
+<h2 class="todo_group">
+  <a name="statistics">Overall Statistics</a>
+  <span class="todo_stats">$stats</span>
+</h2>
+
+<table class="todo_stats">
+END_STATISTICS_HEADER
+}
+
+# Return the statistics footer:
+sub compose_statistics_footer {
+  # Return the table closing:
+  return "</table>\n\n";
+}
+
+# Return the group statistics entry:
+sub compose_statistics_entry {
+  my ($group, $percentage, $done, $tasks, $total) = @_;
+
+  # Count the progress bar width:
+  my $width = $percentage ? $percentage * 2 : 1;
+
+  # Decide which class to use:
+  my $class = $total ? 'todo_bluebar' : 'todo_greenbar';
+
+  # Return the statistics entry:
+  return << "END_STATISTICS_ENTRY";
+  <tr>
+    <td class="todo_group">$group</td>
+    <td class="todo_progress">
+      <div class="$class" style="width: ${width}px;">&nbsp;</div>
+      <span title="$done from $tasks">$percentage&nbsp;%</span>
+    </td>
+  </tr>
+END_STATISTICS_ENTRY
+}
+
 # Display usage information:
 sub display_help {
   my $NAME = NAME;
@@ -345,6 +616,7 @@ Additional options:
   -D, --no-date            do not include due date column
   -P, --no-priority        do not include priority column
   -S, --no-state           do not include state column
+  --with-stats, --stats    include overall task list statistics
 END_HELP
 
   # Return success:
@@ -370,186 +642,6 @@ END_VERSION
   return 1;
 }
 
-# Create a style sheet file and return the LINK element:
-sub write_style_sheet {
-  # Do not create the style sheet when writing to STDOUT:
-  return 0 if $outfile eq '-';
-
-  # Derive the style sheet file name:
-  (my $file = $outfile) =~ s/(\.html?|\.php|)$/.css/;
-
-  # Do not rewrite existing style sheet if its preservation is requested:
-  unless (-e $file && $preserve) {
-    # Open the file for writing:
-    if (open(STYLE, ">$file")) {
-      # Write the document structure related part:
-      print STYLE $css_structure unless $bare;
-
-      # Write the tasks related part:
-      print STYLE $css_tasks;
-
-      # Close the file:
-      close(STYLE);
-    }
-    else {
-      # Report failure:
-      print STDERR "Unable to write to `$file'.\n";
-
-      # Return failure:
-      return 0;
-    }
-  }
-
-  # Return the LINK element:
-  return "<link rel=\"stylesheet\" href=\"" . basename($file) .
-         "\" type=\"text/css\">\n";
-}
-
-# Return the style sheet or a LINK element pointing to it:
-sub style_sheet {
-  # Check whether to have style sheet as a separate file:
-  unless ($inline) {
-    # Return the LINK element:
-    return write_style_sheet();
-  }
-  else {
-    # Return the style sheet:
-    return << "END_STYLE_SHEET";
-<style type="text/css"><!--
-$css_structure$css_tasks  --></style>
-END_STYLE_SHEET
-  }
-}
-
-# Return the HTML header:
-sub html_header {
-  my ($NAME, $VERSION) = (NAME, VERSION);
-
-  # Initialize required variables:
-  my $style_sheet      = style_sheet() || "\n";
-  my $timestamp        = localtime(time);
-
-  # Return the HTML document beginning:
-  return << "END_HTML_HEADER";
-<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01//EN"
-                      "http://www.w3.org/TR/html4/strict.dtd">
-<html>
-<head>
-  <meta http-equiv="Content-Type" content="text/html; charset=$encoding">
-  <meta name="Generator" content="$NAME, $VERSION">
-  <meta name="Date" content="$timestamp">
-  $style_sheet  <title>$heading</title>
-</head>
-
-<body>
-
-<div id="wrapper">
-
-<div id="heading">
-  <table>
-    <tr>
-      <td>
-        <h1><a href="#">$heading</a></h1>
-        $timestamp
-      </td>
-    </tr>
-  </table>
-</div>
-
-<div id="content">
-
-END_HTML_HEADER
-}
-
-# Return the HTML footer:
-sub html_footer {
-  my ($NAME, $VERSION) = (NAME, VERSION);
-
-  # Return the HTML document closing:
-  return << "END_HTML_FOOTER";
-</div>
-
-<div id="footer">
-  Generated using <a href="http://w2do.blackened.cz/">$NAME $VERSION</a>.
-</div>
-
-</div>
-
-</body>
-</html>
-END_HTML_FOOTER
-}
-
-# Return the group header:
-sub group_header {
-  my ($group, $tasks, $done) = @_;
-
-  # Prepare the group statistics:
-  my $stats = sprintf "%d task%s, %d unfinished",
-                      $tasks, (($tasks != 1) ? 's' : ''),
-                      $tasks - $done;
-
-  # Prepare the table header columns:
-  my $cols = '';
-  $cols .= "    <th class=\"todo_id\">id</th>\n" if $with_id;
-  $cols .= "    <th class=\"todo_date\">due date</th>\n" if $with_date;
-  $cols .= "    <th class=\"todo_priority\">priority</th>\n" if $with_pri;
-  $cols .= "    <th class=\"todo_state\">state</th>\n" if $with_state;
-  $cols .= "    <th class=\"todo_description\">task</th>";
-
-  # Return the group heading and the table header:
-  return << "END_GROUP_HEADER";
-<h2 class="todo_group">
-  <a name="$group">$group</a>
-  <span class="todo_stats">$stats</span>
-</h2>
-
-<table class="todo_tasks">
-  <tr>
-$cols
-  </tr>
-END_GROUP_HEADER
-}
-
-# Return the group footer:
-sub group_footer {
-  # Return the table closing:
-  return "</table>\n\n";
-}
-
-# Return the task entry:
-sub task_entry {
-  my ($id, $date, $priority, $state, $task) = @_;
-
-  # Decide which class the task belongs to:
-  my $class      = ($state eq 't') ? ' class="todo_finished"' : '';
-
-  # Prepare the aliases:
-  my @priorities = ('very high', 'high', 'medium', 'low', 'very low');
-  $state         = ($state eq 't') ? 'ok' : '&nbsp;';
-  $priority      = $priorities[--$priority];
-
-  # Escape reserved characters:
-  $task =~ s/&/&amp;/g;
-  $task =~ s/</&lt;/g;
-  $task =~ s/>/&gt;/g;
-
-  # Prepare the table columns:
-  my $cols = '';
-  $cols .= "    <td class=\"todo_id\">$id</td>\n" if $with_id;
-  $cols .= "    <td class=\"todo_date\">$date</td>\n" if $with_date;
-  $cols .= "    <td class=\"todo_priority\">$priority</td>\n" if $with_pri;
-  $cols .= "    <td class=\"todo_state\">$state</td>\n" if $with_state;
-  $cols .= "    <td class=\"todo_description\">$task</td>";
-
-  # Return the task entry:
-  return << "END_TASK_ENTRY";
-  <tr$class>
-$cols
-  </tr>
-END_TASK_ENTRY
-}
-
 # Write items in the task list to the selected output:
 sub write_tasks {
   my $args  = shift || die 'Missing argument';
@@ -558,16 +650,17 @@ sub write_tasks {
   my $stats = {};
   my @data;
 
+  # Get task list statistics:
+  my ($groups, $tasks, $undone) = get_stats($stats);
+  my  $done = $tasks - $undone;
+
   # Load matching tasks:
   load_selection(\@data, undef, $args);
 
-  # Get task list statistics:
-  get_stats($stats);
-
   # Open the selected output for writing:
   if (open(FILE, ">$outfile")) {
     # Write header:
-    print(FILE html_header()) or return 0 unless $bare;
+    print(FILE compose_html_header()) or return 0 unless $bare;
 
     # Check whether the list is not empty:
     if (@data) {
@@ -581,30 +674,60 @@ sub write_tasks {
         # Write heading when group changes:
         if (lc($1) ne $group) {
           # Write group closing if opened:
-          print FILE group_footer() if $group;
+          print FILE compose_group_footer() if $group;
 
           # Translate the group name to lower case:
           $group = lc($1);
 
           # Write group beginning:
-          print FILE group_header($1, $stats->{$group}->{tasks},
-                                      $stats->{$group}->{done});
+          print FILE compose_group_header($1, $stats->{$group}->{tasks},
+                                              $stats->{$group}->{done});
         }
 
         # Write task entry:
-        print FILE task_entry($6, $2, $3, $4, $5);
+        print FILE compose_task_entry($6, $2, $3, $4, $5);
       }
 
       # Write group closing:
-      print FILE group_footer();
+      print FILE compose_group_footer();
     }
     else {
       # Report an empty list:
-      print FILE "<p>The task list is empty.</p>\n\n";
+      print FILE "<p>No matching task found.</p>\n\n";
+    }
+
+    # Check whether to include overall statistics:
+    if ($groups && $with_stats) {
+      # Write statistics beginning:
+      print FILE compose_statistics_header($groups, $tasks, $undone);
+
+      # Process each group:
+      foreach my $group (sort (keys %$stats)) {
+        # Get the group statistics:
+        my $group_done = $stats->{$group}->{done};
+        my $group_all  = $stats->{$group}->{tasks};
+
+        # Count the percentage:
+        my $percentage = int($group_done * 100 / $group_all);
+
+        # Write the group progress entry:
+        print FILE compose_statistics_entry($group, $percentage,
+                                            $group_done, $group_all);
+      }
+
+      # Count the overall percentage:
+      my $percentage = $tasks ? int($done * 100 / $tasks) : 0;
+
+      # Write the overall progress entry:
+      print FILE compose_statistics_entry('total', $percentage, $done,
+                                          $tasks, 1);
+
+      # Write statistics closing:
+      print FILE compose_statistics_footer();
     }
 
     # Write footer:
-    print FILE html_footer() unless $bare;
+    print FILE compose_html_footer() unless $bare;
 
     # Close the outpt:
     close(FILE);
@@ -696,34 +819,36 @@ Getopt::Long::Configure('no_auto_abbrev', 'no_ignore_case', 'bundling');
 # Parse command line options:
 GetOptions(
   # General options:
-  'help|h'         => sub { display_help();    exit 0 },
-  'version|v'      => sub { display_version(); exit 0 },
+  'help|h'                => sub { display_help();    exit 0 },
+  'version|v'             => sub { display_version(); exit 0 },
 
   # Specifying options:
-  'task|t=s'       => sub { $args{task}     = $_[1] },
-  'group|g=s'      => sub { $args{group}    = $_[1] },
-  'date|d=s'       => sub { $args{date}     = $_[1] },
-  'priority|p=i'   => sub { $args{priority} = $_[1] },
-  'finished|f'     => sub { $args{state}    = 't' },
-  'unfinished|u'   => sub { $args{state}    = 'f' },
+  'task|t=s'              => sub { $args{task}     = $_[1] },
+  'group|g=s'             => sub { $args{group}    = $_[1] },
+  'date|d=s'              => sub { $args{date}     = $_[1] },
+  'priority|p=i'          => sub { $args{priority} = $_[1] },
+  'finished|f'            => sub { $args{state}    = 't' },
+  'unfinished|u'          => sub { $args{state}    = 'f' },
 
   # Additional options:
-  'savefile|s=s'   => sub { $savefile       = $_[1] },
-  'output|o=s'     => sub { $outfile        = $_[1] },
-  'encoding|e=s'   => sub { $encoding       = $_[1] },
-  'heading|H=s'    => sub { $heading        = $_[1] },
-  'preserve|k'     => sub { $preserve       = 1 },
-  'inline|i'       => sub { $inline         = 1 },
-  'no-bare|B'      => sub { $bare           = 0 },
-  'bare|b'         => sub { $bare           = 1 },
-  'no-id|I'        => sub { $with_id        = 0 },
-  'with-id'        => sub { $with_id        = 1 },
-  'no-date|D'      => sub { $with_date      = 0 },
-  'with-date'      => sub { $with_date      = 1 },
-  'no-priority|P'  => sub { $with_pri       = 0 },
-  'with-priority'  => sub { $with_pri       = 1 },
-  'no-state|S'     => sub { $with_state     = 0 },
-  'with-state'     => sub { $with_state     = 1 },
+  'savefile|s=s'          => sub { $savefile       = $_[1] },
+  'output|o=s'            => sub { $outfile        = $_[1] },
+  'encoding|e=s'          => sub { $encoding       = $_[1] },
+  'heading|H=s'           => sub { $heading        = $_[1] },
+  'preserve|k'            => sub { $preserve       = 1 },
+  'inline|i'              => sub { $inline         = 1 },
+  'no-bare|B'             => sub { $bare           = 0 },
+  'bare|b'                => sub { $bare           = 1 },
+  'no-id|I'               => sub { $with_id        = 0 },
+  'with-id'               => sub { $with_id        = 1 },
+  'no-date|D'             => sub { $with_date      = 0 },
+  'with-date'             => sub { $with_date      = 1 },
+  'no-priority|P'         => sub { $with_pri       = 0 },
+  'with-priority'         => sub { $with_pri       = 1 },
+  'no-state|S'            => sub { $with_state     = 0 },
+  'with-state'            => sub { $with_state     = 1 },
+  'no-stats|no-stat'      => sub { $with_stats     = 0 },
+  'stats|stat|with-stats' => sub { $with_stats     = 1 },
 );
 
 # Detect superfluous options:
@@ -905,6 +1030,14 @@ Do not include state column in the listing.
 
 Include state column in the listing; the default option.
 
+=item B<--with-stats>, B<--stats>
+
+Include overall task list statistics.
+
+=item B<--no-stats>
+
+Do not include overall task list statistics; the default option.
+
 =back
 
 =head1 ENVIRONMENT
